/* Zadatak 5:

*/

#include <stdio.h>

/* ========================================================================= */

    /* Funkcija odobreni. */

    /* Skica algoritma:
       - Vanjska petlja ide po i (unaprijed ili unatrag, kako nam pase).
         Za fiksni i,
         - Izracunamo sve potrebne vrijednosti P_(i, m)(x) = petlja po m,
           i usput provjeravamo je li a[i] zlocest.
         - Ako za neki m dobijemo P_(i, m)(x) < a[i], tj. a[i] je zlocest,
           onda mozemo prekinuti petlju po m.
         - Na kraju, ako je a[i] zlocest, izbacimo ga iz polja a.
       Zbog izbacivanja i pomaka, zgodnije je da petlja po i ide SILAZNO.
       Tada pomicemo samo vec provjerene clanove polja (od i+1 do n-1),
       a sljedeci clan za provjeru = a_(i-1) je ISPRED njih. Dakle,
       samo treba smanjiti i za 1 u petlji. Zato silazno!

       Ostaje pogledati racunanje vrijednosti P_(i, m), za razne m.
       Prvih nekoliko vrijednosti polinoma je:
           P_(i, 0)(x) = a_i,
           P_(i, 1)(x) = a_(i-1) + a_i * x
           P_(i, 2)(x) = a_(i-2) + a_(i-1) * x + a_i * x^2
           ...
       Odavde slijedi (raspisemo zadanu formulu)
           P_(i, m)(x) = x * P_(i, m-1)(x) + a_(i-m),  za m >= 1,
       uz pocetak P_(i, 0)(x) = a_i.
       Algoritam za racunanje je upravo Hornerov algoritam, samo sto
       petlja po m ide uzlazno.
         Radi jednostavnosti, petlja po m ide od 0 (a NE od 1), do
       m_max = min(d, i), a pocetna vrijednost je onda P(i, -1)(x) = 0.
    */

int odobreni(int n, int d, double x, double a[])
{
        /* Pocetna gornja granica unutarnje petlje po m je m_max = d.
           Ako (ili kad) i padne ispod d, smanjujemo granicu na i. */
    int m_max = d;
    int i, j, m, zlocest;
    double P_im;


        /* Vanjska petlja po i je SILAZNA, zbog izbacivanja. */
    for (i = n - 1; i >= 0; --i) {

            /* Ako je i < m_max, smanji gornju granicu petlje po m.
               Tu koristimo da i pada! */
        if (i < m_max) m_max = i;

            /* Inicijalizacija "je li a[i] zlocest".
               Trenutno (prije provjere), nije zlocest. */
        zlocest = 0;

            /* Inicijalizacija za Hornerov algoritam.
               Prva vrijednost je P_(i, -1)(x) = 0. */
        P_im = 0.0;

            /* Uzlazna petlja po m, od 0 do m_max.
               Hornerov algoritam za racunanje P_(i, m)(x). */
        for (m = 0; m <= m_max; ++m) {

                /* Nova vrijednost P_(i, m)(x) iz prethodne. */
            P_im = x * P_im + a[i - m];

                /* Provjera je li a[i] zlocest.
                   Ako je, odmah prekidamo petlju po m. */
            if (P_im < a[i]) {
                zlocest = 1;
                break;
            }
        }

            /* Ako je a[i] zlocest, izbaci ga iz polja a. */
        if (zlocest) {
                /* Odmah smanji n za 1. */
            --n;
                /* Pomakni clanove a[i+1], ..., a[novi n]
                   za jedno mjesto prema dolje (lijevo). */
            for (j = i; j < n; ++j)
                a[j] = a[j + 1];
        }
    }    /* Kraj petlje po i. */

        /* Vrati novi duljinu polja nakon izbacivanja. */
    return n;
}

/* ========================================================================= */

    /* Pomocna funkcija za ispis polja. */

void print_polje(int n, double a[])
{
    int i;
    for (i = 0; i < n; i++)
        printf("  %g", a[i]);
    printf("\n");
    return;
}

/* ========================================================================= */

    /* Glavni program za demo. */

int main(void)
{
        /* Polazno polje a sa 6 elemenata. */
    int n = 6;
    double a[6] = { 1.0, -1.0, -2.0, 3.0, 1.0, -2.0 };

        /* Zadani maksimalni stupanj d. */
    int d = 1;

        /* Zadana tocka x. */
    double x = 1.0;

    printf("Pocetno polje, duljine n = %d:\n", n);
    print_polje(n, a);

    n = odobreni(n, d, x, a);
    printf("Polje nakon odobreni, duljine n = %d:\n", n);
    print_polje(n, a);

    return 0;
}
